/**************************************************************************
 
 	main_menu.c
 
  	Project: RockBlast an asteroid game. 
 
	Copyright (c) Febuary 2001 Garett Spencley
 
   	This program is free software; you can redistribute it and/or modify
   	it under the terms of the GNU General Public License as published by
   	the Free Software Foundation; either version 2, or (at your option)
   	any later version.

   	This program is distributed in the hope that it will be useful,
   	but WITHOUT ANY WARRANTY; without even the implied warranty of
   	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   	GNU General Public License for more details.

   	You should have received a copy of the GNU General Public License
   	along with this program; if not, write to the Free Software Foundation,
   	Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 

****************************************************************************/

#include "main_menu.h"
#include "image.h"

#define OPTION_X_POS (screen->w / 2) - 200

enum {
	OP_START,
	OP_QUIT,
	NUM_OPTIONS
};

option_t options[NUM_OPTIONS];

object logo;

bool START_GAME;

inline void update_menu_screen();
void free_menu_data();
bool load_menu_data();

void erase_option(unsigned int);
void draw_option(unsigned int);

void menu_handle_key(SDLKey key);

/* 
 * load_menu_data() - load data for menu
 */

bool load_menu_data()
{
	SDL_Surface *background;

	background = load_image(DATAFILE("background.png"), false);
	if(!background) {
		fprintf(stderr, "%s\n", SDL_GetError());
		return false;
	}

	if(!set_background(background)) {
		fprintf(stderr, "%s\n", SDL_GetError());
		return false;
	}

	SDL_FreeSurface(background);

	logo.image = load_image(DATAFILE("rockblast.png"), true);
	if(!logo.image) {
		fprintf(stderr, "%s\n", SDL_GetError());
		return false;
	}
	logo.x = (screen->w - logo.image->w) / 2;
	logo.y = 10;

	draw_object(&logo);

	/* TODO: Error checking! */
	options[OP_START].selected_image = load_image(DATAFILE("start_sel.png"), true);
	options[OP_START].reg_image = load_image(DATAFILE("start_reg.png"), true);

	options[OP_QUIT].selected_image = load_image(DATAFILE("quit_sel.png"), true);
	options[OP_QUIT].reg_image = load_image(DATAFILE("quit_reg.png"), true);
		
	return true;
}

/* 
 * init_options() - initialize option positions
 */

void init_options()
{
	int i;

	options[OP_START].obj.x = OPTION_X_POS;
	options[OP_START].obj.y = 200;

	options[OP_START].selected = true;

	options[OP_QUIT].obj.x = OPTION_X_POS;
	options[OP_QUIT].obj.y = options[OP_START].obj.y + options[OP_START].reg_image->h + 50;

	options[OP_QUIT].selected = false;

	for(i = 0; i < NUM_OPTIONS; i++) 
		draw_option(i);
}

/*
 * free_menu_data() - free memory used by menu data
 */

void free_menu_data()
{
	int i;

	for(i = 0; i < NUM_OPTIONS; i++) {
		SDL_FreeSurface(options[i].selected_image);
		SDL_FreeSurface(options[i].reg_image);
	}
	
	SDL_FreeSurface(logo.image);
}

/* 
 * erase_option - erase an option object 
 */

void erase_option(unsigned int opt)
{
	if(opt >= NUM_OPTIONS)
		return;

	erase_object(&options[opt].obj);
}

/*
 * draw_option() - draw an option object
 */

void draw_option(unsigned int opt)
{
	if(opt >= NUM_OPTIONS)
		return;

	if(options[opt].selected)
		options[opt].obj.image = options[opt].selected_image;
	else 
		options[opt].obj.image = options[opt].reg_image;

	draw_object(&options[opt].obj);
}

/* 
 * menu() - perform menu loop
 */

void main_menu()
{
	SDL_Event event;
	int i;
		
	if(!load_menu_data()) {
		fprintf(stderr, "%s\n", SDL_GetError());
		return;
	}

	paint_background();

	init_options();

	/* main loop */

	START_GAME = false;

	flush_event_queue();

	while(!START_GAME && !QUIT) {

		wait_frame(32);

		while(SDL_PollEvent(&event)) {

			if(event.type == SDL_QUIT) 
				QUIT = true;

			if(event.type == SDL_KEYDOWN)
				menu_handle_key(event.key.keysym.sym);
		}

		erase_object(&logo);

		for(i = 0; i < NUM_OPTIONS; i++)
			erase_option(i);

		for(i = 0; i < NUM_OPTIONS; i++)
			draw_option(i);

		draw_object(&logo);

		update_screen();
	}

	free_menu_data();
	return;
}

/*
 * handle_option() - handle an option
 */

void handle_option(unsigned int opt) 
{
	switch(opt) {

	case OP_START:
		START_GAME = true;
		break;
	
	case OP_QUIT:
		QUIT = true;
		break;

	default:
		break;
	}

	return;
}

/*
 * menu_handle_key() - handle a key event
 */

void menu_handle_key(SDLKey key)
{
	int i;
	
	if(key == SDLK_ESCAPE) {
		QUIT = true;
		return;
	}

	if(key == SDLK_RETURN || key == SDLK_SPACE) {

		for(i = 0; i < NUM_OPTIONS; i++) {

			if(options[i].selected) {
				handle_option(i);
				break;
			}
		}
	}
	
	if(key == SDLK_DOWN) {
			
		for(i = 0; i < NUM_OPTIONS; i++) {
				
			if(options[i].selected == true) {
					
				options[i].selected = false;
				
				if(i == (NUM_OPTIONS-1))
					options[0].selected = true;
				else
					options[i+1].selected = true;
				
				break;
			}
		}	
	}

	if(key == SDLK_UP) {

		for(i = 0; i < NUM_OPTIONS; i++) {
				
			if(options[i].selected == true) {
					
				options[i].selected = false;
				
				if(i == 0)
					options[NUM_OPTIONS-1].selected = true;
				else
					options[i-1].selected = true;
				
				break;
			}
		}	
	}

}

