/**************************************************************************
 
 	video.c
 
  	Project: RockBlast an asteroid game. 
 
	Copyright (c) Febuary 2001 Garett Spencley
 
   	This program is free software; you can redistribute it and/or modify
   	it under the terms of the GNU General Public License as published by
   	the Free Software Foundation; either version 2, or (at your option)
   	any later version.

   	This program is distributed in the hope that it will be useful,
   	but WITHOUT ANY WARRANTY; without even the implied warranty of
   	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   	GNU General Public License for more details.

   	You should have received a copy of the GNU General Public License
   	along with this program; if not, write to the Free Software Foundation,
   	Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 

****************************************************************************/

#include "video.h"
#include "image.h"

SDL_Surface *screen     = NULL;
SDL_Surface *background = NULL;

blit blits[MAX_UPDATES];
int num_updates;

SDL_Rect srcupdate[MAX_UPDATES];
SDL_Rect dstupdate[MAX_UPDATES];

inline void add_object(object *, blit *, int *);
inline void delete_object(object *, blit *, int *, SDL_Surface *);

/*
 * init_video() - Initialize video mode and window. 
 */

bool init_video()
{
	if(SDL_Init(SDL_INIT_VIDEO) < 0)
		return false;

	screen = SDL_SetVideoMode(MODE_X, MODE_Y, BPP, VIDEO_FLAGS);
	if(!screen)
		return false;

	SDL_WM_SetCaption(RB_APP_TITLE, NULL);
	SDL_ShowCursor(0);

	init_blits();
	return true;
}

/*
 * init_blits() - initialize blits array
 */

void init_blits() 
{
	int i;

	for(i = 0; i < MAX_UPDATES; i++) {
		blits[i].srcrect = &srcupdate[i];		
		blits[i].dstrect = &dstupdate[i];		
	}	

	num_updates = 0;
}

void free_video()
{
	SDL_FreeSurface(background);
}

/*
 * video_resize() - resize the screen
 */

SDL_Surface *video_resize(int w, int h)
{
	return SDL_SetVideoMode(w, h, BPP, SDL_RESIZABLE);
}

/* 
 * wait_frame()
 */

void wait_frame(int frames_per_sec)
{
	static Uint32 next_tick = 0;
	Uint32 this_tick;

	/* Wait for the next frame */
	this_tick = SDL_GetTicks(); 
	
	if(this_tick < next_tick) 
		SDL_Delay(next_tick - this_tick);
	
	next_tick = this_tick  + (1000 / frames_per_sec);
}

/*
 * update_screen() - redraws the screen
 */

inline void update_screen()
{
	int i;

	for(i = 0; i < num_updates; i++) 
		SDL_LowerBlit(blits[i].src, blits[i].srcrect, screen, blits[i].dstrect);
	
	SDL_UpdateRects(screen, num_updates, dstupdate);
	num_updates = 0;
}

/* 
 * draw_object() - draws an object
 */

void draw_object(object *obj)
{
	add_object(obj, blits, &num_updates);
}

/*
 * erase_object() - erases an object
 */

void erase_object(object *obj)
{
	delete_object(obj, blits, &num_updates, background);
}

/*
 * set_background() - set the background
 */

bool set_background(SDL_Surface *back)
{
	if(!back) 
		return false;
	
	if(background != NULL)
		SDL_FreeSurface(background);

	background = SDL_ConvertSurface(back, back->format, back->flags);
	if(!background) 
		return false;

	return true;
}

/*
 * paint_background() - paint the background
 */

void paint_background(void) 
{
	SDL_Rect dst;

	dst.x = 0;
	dst.y = 0;
	dst.w = background->w;
	dst.h = background->h;

	SDL_BlitSurface(background, NULL, screen, &dst);
	SDL_UpdateRect(screen, 0, 0, 0, 0);
}

/*
 * add_object() - adds an object to the blit array 
 */

inline void add_object(object *sprite, blit *blits, int *numupdates)
{
	blit *update;
	
	update = &blits[(*numupdates)++];
	
	update->src = sprite->image;
	update->srcrect->x = 0;
	update->srcrect->y = 0;
	update->srcrect->w = sprite->image->w;
	update->srcrect->h = sprite->image->h;
	
	update->dstrect->x = sprite->x;
	update->dstrect->y = sprite->y;
	update->dstrect->w = sprite->image->w;
	update->dstrect->h = sprite->image->h;
}

/*
 * delete_object() - removes an object from the blit array
 */

inline void delete_object(object *sprite, blit *blits, int *numupdates,
	SDL_Surface *background)
{
	blit *update;

	update = &blits[(*numupdates)++];

	update->src = background;
	update->srcrect->x = sprite->x % background->w;
	update->srcrect->y = sprite->y;
	update->srcrect->w = sprite->image->w;
	update->srcrect->h = sprite->image->h;
	
	update->dstrect->x = sprite->x;
	update->dstrect->y = sprite->y;
	update->dstrect->w = update->srcrect->w;
	update->dstrect->h = update->srcrect->h;
	
}
